﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.Gui.Wpf
{
   /// <summary>
   /// Interaction logic for BibleView.xaml
   /// </summary>
   public partial class BibleView : UserControl, IBibleView
   {
      FilteredView _searchResultsWidget;
      PassageView _passageView;
      GenericView _genericView;
      FilteredView _booksView;
      TabTitle _tabTitle;
      bool _completionInitalised;
      IEnumerable<string> _books;

      public BibleView()
      {
         InitializeComponent();

         _searchResultsWidget = new FilteredView();

         _passageView = new PassageView();

         _genericView = new GenericView();

         _booksView = new FilteredView();

         _searchBox.TextChanged += SearchBox_TextChanged;
         _searchBox.LostFocus += SearchBox_LostFocus;
         _searchBox.LostMouseCapture += SearchBox_LostMouseCapture;
         _searchBox.LostKeyboardFocus += SearchBox_LostKeyboardFocus;
         _searchBox.KeyUp += SearchBox_KeyUp;
         this.Loaded += BibleView_Loaded;

         _settingsButton.LostFocus += SettingsButton_LostFocus;
         _settingsButton.LostMouseCapture += SettingsButton_LostMouseCapture;
         _settingsButton.LostKeyboardFocus += _settingsButton_LostKeyboardFocus;
      }

      /// <summary>
      /// The tab page this is being shown in.
      /// </summary>
      internal TabItem TabPage
      {
         get;
         set;
      }

      void _settingsButton_LostKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
      {
         _settingsPopup.IsOpen = false;
      }

      void SettingsButton_LostMouseCapture(object sender, MouseEventArgs e)
      {
         _settingsPopup.IsOpen = false;
      }

      void SettingsButton_LostFocus(object sender, RoutedEventArgs e)
      {
         _settingsPopup.IsOpen = false;
      }

      void SelectNextItem()
      {
         int selectedIndex = _autoCompleteList.SelectedIndex + 1;
         _autoCompleteList.SelectedIndex = selectedIndex;
         if (selectedIndex >= _autoCompleteList.Items.Count)
         {
            _autoCompleteList.SelectedIndex = 0;
         }
      }

      void SelectPreviousItem()
      {
         int selectedIndex = _autoCompleteList.SelectedIndex - 1;
         if (selectedIndex < 0)
         {
            _autoCompleteList.SelectedIndex = _autoCompleteList.Items.Count - 1;
            return;
         }
         _autoCompleteList.SelectedIndex = selectedIndex;
      }

      void SearchBox_KeyUp(object sender, KeyEventArgs e)
      {
         if (e.Key == Key.Down)
         {
            SelectNextItem();
            return;
         }
         if (e.Key == Key.Up)
         {
            SelectPreviousItem();
            return;
         }
         if (e.Key == Key.Enter)
         {
            var item = _autoCompleteList.SelectedItem;
            if (item != null)
            {
               _searchBox.Text = ((Label)item).Content.ToString();
               _autoComplete.IsOpen = false;
            }
         }
      }

      void BibleView_Loaded(object sender, RoutedEventArgs e)
      {
         _searchBox.Focus();
         var window = Window.GetWindow(this);
         window.LocationChanged += Window_LocationChanged;
         window.SizeChanged += Window_SizeChanged;
      }

      void Window_SizeChanged(object sender, SizeChangedEventArgs e)
      {
         _autoComplete.IsOpen = false;
         _settingsPopup.IsOpen = false;
      }

      void Window_LocationChanged(object sender, EventArgs e)
      {
         _autoComplete.IsOpen = false;
         _settingsPopup.IsOpen = false;
      }

      void SearchBox_LostKeyboardFocus(object sender, KeyboardFocusChangedEventArgs e)
      {
         _autoComplete.IsOpen = false;
      }

      void SearchBox_LostMouseCapture(object sender, MouseEventArgs e)
      {
         _autoComplete.IsOpen = false;
      }

      void SearchBox_LostFocus(object sender, RoutedEventArgs e)
      {
         _autoComplete.IsOpen = false;
      }

      void SearchBox_TextChanged(object sender, TextChangedEventArgs e)
      {
         if (!_completionInitalised)
         {
            RequestAutoCompleteList.Fire(this, new EventArgs());
            _completionInitalised = true;
         }

         _autoComplete.IsOpen = true;
         var searchText = _searchBox.Text;
         _autoCompleteList.Items.Clear();
         searchText = searchText.ToLower();
         var matchingBooksQuery =
             from book in _books
             let bookLower = book.ToLower()
             where bookLower.StartsWith(searchText) && bookLower != searchText
             select book;

         var matchingBooks = matchingBooksQuery.ToArray();

         bool isOpen = !(searchText == string.Empty) && matchingBooks.Length > 0;
         if (isOpen)
         {
            _autoComplete.Width = _searchBox.ActualWidth;
         }
         _autoComplete.IsOpen = isOpen;

         foreach (var book in matchingBooks)
         {
            var label = new Label() { Content = book };
            label.MouseDown += Label_MouseDown;
            int itemNumber = _autoCompleteList.Items.Add(label);
         }

         SearchRequest.Fire(this, new StringEventArgs(searchText));
      }

      void Label_MouseDown(object sender, MouseButtonEventArgs e)
      {
         _searchBox.Text = ((Label)sender).Content.ToString();
         _autoComplete.IsOpen = false;
      }

      public event EventHandler<EventArgs> NavigateBackwards;

      public event EventHandler<EventArgs> NavigateForwards;

      public event EventHandler<StringEventArgs> SearchRequest;

      public event EventHandler<EventArgs> RequestAutoCompleteList;

      public event EventHandler<EventArgs> RequestAboutInfo;

      public event EventHandler<EventArgs> Browse;

      public IPassageView PassageView
      {
         get
         {
            return _passageView;
         }
      }

      public IFilteredView SearchView
      {
         get
         {
            return _searchResultsWidget;
         }
      }

      public IGenericView GenericView
      {
         get
         {
            return _genericView;
         }
      }

      public IFilteredView BooksView
      {
         get
         {
            return _booksView;
         }
      }

      public void ShowPassageView()
      {
         ThreadSafeShow(_passageView);
      }

      public void ShowSearchView()
      {
         ThreadSafeShow(_searchResultsWidget);
      }

      public void ShowGenericView()
      {
         ThreadSafeShow(_genericView);
      }

      private void ThreadSafeShow(UserControl userControl)
      {
         Dispatcher.Invoke((Action)(() =>
         {
            ShowWidget(userControl);
         }));
      }

      UserControl _currentControl;

      void ShowWidget(UserControl userControl)
      {
         if (_currentControl == userControl)
         {
            return;
         }
         _grid.Children.Remove(_currentControl);
         _grid.Children.Add(userControl);
         Grid.SetRow(userControl, 1);
         Grid.SetColumnSpan(userControl, 4);
         _currentControl = userControl;
      }

      public void ShowBooksView()
      {
         ThreadSafeShow(_booksView);
      }

      public UserControl CreateTabTitle()
      {
         _tabTitle = new TabTitle();
         _tabTitle.CloseClicked += TabTitle_CloseClicked;
         return _tabTitle;
      }

      internal event EventHandler<EventArgs> TabCloseRequested;

      void TabTitle_CloseClicked(object sender, EventArgs e)
      {
         TabCloseRequested.Fire(this, e);
      }

      public void ChangeTitle(string title)
      {
         Dispatcher.Invoke((Action)(() =>
         {
            _tabTitle.Text = title;
         }));
      }

      public void ChangeBibleBar(string text)
      {
         throw new NotImplementedException();
      }

      public void ShowAutoCompleteList(IEnumerable<string> list)
      {
         _books = list;
      }

      public void ShowAboutInfo(string text)
      {
         throw new NotImplementedException();
      }

      public void RunOnUiThread(Action action)
      {
         Dispatcher.Invoke(action);
      }

      public bool Editable
      {
         get
         {
            return this.IsEnabled;
         }
         set
         {
            Dispatcher.Invoke((Action)(() =>
            {
               this.IsEnabled = value;
            }));
         }
      }

      void BackButton_Click(object sender, RoutedEventArgs e)
      {
         NavigateBackwards.Fire(this, EventArgs.Empty);
      }

      void ForwardButton_Click(object sender, RoutedEventArgs e)
      {
         NavigateForwards.Fire(this, EventArgs.Empty);
      }

      void MenuButton_Click(object sender, RoutedEventArgs e)
      {
         _settingsPopup.IsOpen = true;
      }

      void MenuItemAbout_Click(object sender, RoutedEventArgs e)
      {
         RequestAboutInfo.Fire(this, EventArgs.Empty);
      }

      void BrowseButton_Click(object sender, RoutedEventArgs e)
      {
         Browse.Fire(this, EventArgs.Empty);
      }

      CustomPopupPlacement[] SettingsPopupPlacement(Size popupSize, Size targetSize, Point offset)
	   {
		   var placements = new[] { new CustomPopupPlacement() };
         placements[0].Point = new Point(offset.X - popupSize.Width + targetSize.Width, offset.Y + targetSize.Height);
		   return placements;
	   }
   }
}
