﻿using SharpSword;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WideMargin.MVCInterfaces;
using WideMargin.Utilities;

namespace WideMargin.Gui.Wpf
{
    /// <summary>
    /// Interaction logic for PassageView.xaml
    /// </summary>
    public partial class PassageView : UserControl, IPassageView
    {
        public PassageView()
        {
            InitializeComponent();
            _genericView.RequestVersification += WebViewRequestVersification;
            _genericView.VerseLinkClicked += WebViewVerseLinkClicked;
        }

        void WebViewVerseLinkClicked(object sender, LinkClickedEventArgs<IVerseIdentifier> e)
        {
            VerseLinkClicked.Fire(this, e);
        }

        void WebViewRequestVersification(object sender, RequestVersificationEventArgs e)
        {
            RequestVersification.Fire(this, e);
        }

        public event EventHandler<EventArgs> RequestNext;

        public event EventHandler<EventArgs> RequestPrevious;

        public void ShowVerses(IEnumerable<SharpSword.IVerse> searchResults)
        {
            var passageBuilder = new StringBuilder();
            var formatter = SharpSword.Formatters.FormattersManager.Html;
            passageBuilder.Append(formatter.Header());
            foreach (IVerse verse in searchResults)
            {
                passageBuilder.Append(" ");
                passageBuilder.Append(verse.Text);
            }
            passageBuilder.Append(formatter.Footer());

            this.Dispatcher.Invoke((Action)(() =>
            {
                _genericView.Text = passageBuilder.ToString();
            }));
        }

        public void ScrollToVerse(IVerse verse)
        {
            var script = string.Format("scrollTo('{0}');", verse.Identifier);

            this.Dispatcher.Invoke((Action)(() =>
            {
                _genericView.ExecuteScript("scrollTo", verse.Identifier.ToString());
            }));
        }

        public void AppendText(string text)
        {
            throw new NotImplementedException();
        }

        public string Text
        {
            set 
            {
                _genericView.Text = value; 
            }
        }

        public event EventHandler<LinkClickedEventArgs<SharpSword.IVerseIdentifier>> VerseLinkClicked;

        public event EventHandler<LinkClickedEventArgs<string>> ActionLinkClicked;

        public event EventHandler<RequestVersificationEventArgs> RequestVersification;

        void PrevButton_Click(object sender, RoutedEventArgs e)
        {
            RequestPrevious.Fire(this, EventArgs.Empty);
        }

        void NextButton_Click(object sender, RoutedEventArgs e)
        {
            RequestNext.Fire(this, EventArgs.Empty);
        }
    }
}
